library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library WORK;
use WORK.config.all;


entity Top is
	PORT(
		-- GENERIC IOs
		CLK		: in  std_logic;
		RESET	: in  std_logic;

		-- SPI IOs
		SPI_CLK		: in  std_logic;
		SPI_MISO	: out std_logic;
		SPI_MOSI	: in  std_logic;
		SPI_SS		: in  std_logic;

		O			: out std_logic
	);
end entity Top;


architecture Behavioral of Top is

	component PulseGen
		PORT(
			-- GENERIC IOs
			CLK		: in std_logic;
			CE		: in std_logic;
			RESET	: in std_logic;
			O			: out std_logic;

			-- CONFIG IOs
			CONF_MODE	: in std_logic;
			WE				: in std_logic;
			DAT_W			: in std_logic_vector (31 downto 0);
			ADR				: in integer range 0 to (RAM_SIZE/4)-1;
			DAT_SIZE	: in integer range 0 to (RAM_SIZE/4)-1
		);
	end component;

	component SpiCtrl
		PORT(
			-- GENERIC IOs
			CLK		: in  std_logic;
			CE		: in  std_logic;
			RESET	: in  std_logic;

			-- SPI IOs
			SPI_CLK		: in  std_logic;
			SPI_MISO	: out std_logic;
			SPI_MOSI	: in  std_logic;
			SPI_SS		: in  std_logic;

			-- INTERNAL IOs
			DAT_TX		: in  std_logic_vector (31 downto 0);
			DAT_RX		: out std_logic_vector (31 downto 0);
			TRANS_DONE: out std_logic;
			DAT_VALID	: in  std_logic
		);
	end component;

	component ConfProto
		PORT(
			-- GENERIC IOs
			CLK		: in  std_logic;
			CE		: in  std_logic;
			RESET	: in  std_logic;

			-- INTERNAL SPI IOs
			DAT_TX		: out std_logic_vector (31 downto 0);
			DAT_RX		: in  std_logic_vector (31 downto 0);
			TRANS_DONE: in  std_logic;
			DAT_VALID	: out std_logic;

			-- CONFIG IOs
			CONF_MODE	: out std_logic;
			WE				: out std_logic;
			DAT_W			: out std_logic_vector (31 downto 0);
			ADR				: out integer range 0 to (RAM_SIZE/4)-1;
			DAT_SIZE	: out integer range 0 to (RAM_SIZE/4)-1
		);
	end component;

	signal RESET_INT : std_logic_vector (1 downto 0) := "11";

	signal CONF_MODE : std_logic := '0';
	signal WE : std_logic := '0';
	signal DAT_W : std_logic_vector (31 downto 0) := (others => '0');
	signal ADR : integer range 0 to (RAM_SIZE/4)-1 := 0;
	signal DAT_SIZE : integer range 0 to (RAM_SIZE/4)-1 := 0;

	signal DAT_TX : std_logic_vector (31 downto 0) := (others => '0');
	signal DAT_RX : std_logic_vector (31 downto 0) := (others => '0');
	signal TRANS_DONE : std_logic := '0';
	signal DAT_VALID : std_logic := '0';

	signal CE : std_logic := '0';
	signal COUNTER : integer range 0 to SPI_PRESCALER := 0;

begin

	process (CLK) is
	begin
		if rising_edge(CLK) then
			RESET_INT(0) <= RESET;
			RESET_INT(1) <= RESET_INT(0);
		end if;
	end process;

	process (CLK) is
	begin
		if RESET_INT(1) = '1' then
			COUNTER <= 0;
			CE <= '0';
		elsif rising_edge(CLK) then
			if COUNTER = SPI_PRESCALER then
				COUNTER <= 0;
				CE <= '1';
			else
				COUNTER <= COUNTER + 1;
				CE <= '0';
			end if;
		end if;
	end process;

	SPI : SpiCtrl
		port map(
			CLK => CLK,
			CE => CE,
			RESET => RESET_INT(1),

			SPI_CLK => SPI_CLK,
			SPI_MISO => SPI_MISO,
			SPI_MOSI => SPI_MOSI,
			SPI_SS => SPI_SS,

			DAT_TX => DAT_TX,
			DAT_RX => DAT_RX,
			TRANS_DONE => TRANS_DONE,
			DAT_VALID => DAT_VALID
		);

	PG : PulseGen
		port map(
			CLK => CLK,
			CE => '1', -- This always runs at FCLK.
			RESET => RESET_INT(1),
			O => O,

			CONF_MODE => CONF_MODE,
			WE => WE,
			DAT_W => DAT_W,
			ADR => ADR,
			DAT_SIZE => DAT_SIZE
		);

	Conf : ConfProto
		port map(
			CLK => CLK,
			CE => CE,
			RESET => RESET_INT(1),

			DAT_TX => DAT_TX,
			DAT_RX => DAT_RX,
			TRANS_DONE => TRANS_DONE,
			DAT_VALID => DAT_VALID,

			CONF_MODE => CONF_MODE,
			WE => WE,
			DAT_W => DAT_W,
			ADR => ADR,
			DAT_SIZE => DAT_SIZE
		);

end architecture;
